@@ -8,17 +8,21 @@ use Term::Screen;
 use constant EVENT_SAW_FILE => "\x01";
 use constant EVENT_ENTER_FILE => "\x02";
 use constant EVENT_ENTER_LINE => "\x03";
+use constant EVENT_DIE => "\x04";
 
 my %EVENT = (
     EVENT_SAW_FILE()    => \&_saw_file,
     EVENT_ENTER_FILE()  => \&_enter_file,
     EVENT_ENTER_LINE()  => \&_enter_line,
+    EVENT_DIE()  => \&_throw_exception,
 );
 
 my $screen = Term::Screen->new();
 $screen->clrscr();
 
-for my $accessor (qw(io files current_file_path current_file all_lines num_lines)) { 
+for my $accessor (qw(
+    io files current_file_path current_file all_lines
+    num_lines skip_files last_line skip_installed)) { 
     no strict 'refs'; 
     *{$accessor} = sub { $_[0]->{$accessor}; };
 }
@@ -27,7 +31,13 @@ sub new {
     my ($pkg, $path) = @_;
     
     open my $in, "<", $path or die $!;
-    my $self = bless { io => $in, files => [] }, $pkg;
+    my $self = bless { 
+        io => $in, 
+        files => [], 
+        skip_files => {},
+        last_line => 0,
+        skip_installed => 0,
+    }, $pkg;
 
     return $self;
 }
@@ -49,8 +59,10 @@ sub _enter_file {
     
     close $self->current_file if $self->current_file;
     $self->{current_file_path} = $self->files->[$file_id];
+
     $screen->clrscr();
     $screen->at(0, 0);
+
     if (-e $self->current_file_path) {
         open my $file, "<", $self->current_file_path or die $!;
         $self->{current_file} = $file;
@@ -68,20 +80,27 @@ sub _enter_file {
     1;
 }
 
+my $site_libs = join "|", grep { /^\// } @INC;
+my $site_qr = qr{$site_libs};
+
 sub _enter_line {
     my $self = shift;
+
     my ($buff);
     $self->io->read($buff, 4);
-    my ($line_no) = unpack("L", $buff);
-    
+    my ($line_no) = unpack("L", $buff);    
     $line_no--;
-    
+
+    $self->{last_line} = $line_no;
     return unless $self->current_file;
+    return if $self->skip_files->{$self->current_file_path};
+    return if $self->skip_installed && $self->current_file_path =~ $site_qr;
     
     my $screen_cols = $screen->cols;
+    my $screen_rows = int(($screen->rows - 4) / 2);
     
-    my $from = $line_no > 10 ? $line_no - 10 : 0;
-    my $to = $line_no + 10 < $self->num_lines - 1 ? $line_no + 10 : $self->num_lines - 1;
+    my $from = $line_no > $screen_rows ? $line_no - $screen_rows : 0;
+    my $to = $line_no + $screen_rows < $self->num_lines - 1 ? $line_no + $screen_rows : $self->num_lines - 1;
     
     $screen->at(2, 0);
     $screen->clreos();
@@ -96,8 +115,35 @@ sub _enter_line {
         $screen->normal if $l == $line_no;
     }
 
-    my $chr = $screen->getch();
-    if ($chr eq 'q') { $self->done; }
+    $self->_process_key();
+}
+
+sub _throw_exception {
+    my $self = shift;
+
+    $screen->at(1, 0);
+    $screen->puts("Threw exception at " . $self->last_line . " in " . $self->current_file_path);
+}
+
+my %KEY_HANDLER = (
+    q => sub { shift->done },
+    s => sub { 
+        my $self = shift; 
+        $self->skip_files->{$self->current_file_path} = 1; 
+    },
+    a => sub { shift->{skip_installed} |= 1 },
+);
+
+sub _process_key {
+    my $self = shift;
+    
+    my $k = lc $screen->getch();
+    $screen->at(1, 0);
+
+    my $handler = $KEY_HANDLER{$k} // sub {};
+    $handler->($self);
+    
+    1;
 }
 
 sub playback {
